"""
Image processing utility functions
"""
from math import ceil, floor

import numpy as np
from skimage.exposure import rescale_intensity


def is_mask(img):
    """
    Return True if an image represents a binary mask
    :param img: np.ndarray
    """
    if img.dtype == 'bool':
        return True
    elif img.dtype == 'uint8':
        unique = np.unique(img)
        if unique.shape[0] == 2 and np.all(unique == [0, 255]):
            return True
    return False

def pad(yxcz, mpx: int):
    """
    Pad and crop image data in Y and X axes to meet specific dimension
    :param yxcz: np.ndarray
    :param mpx: int pixel size of resulting square
    :return: np.ndarray array of size (mpx, mpx, nc, nz)
    """
    assert len(yxcz.shape) == 4
    dh = 0.5 * (mpx - yxcz.shape[0])
    dw = 0.5 * (mpx - yxcz.shape[1])

    if dw < 0:
        x0 = floor(-dw)
        x1 = x0 + mpx
        yxcz = yxcz[:, x0:x1, :, :]
        dw = 0
    if dh < 0:
        y0 = floor(-dh)
        y1 = y0 + mpx
        yxcz = yxcz[y0:y1, :, :, :]
        dh = 0

    border = ((floor(dh), ceil(dh)), (floor(dw), ceil(dw)), (0, 0), (0, 0))
    padded = np.pad(yxcz, border, mode='constant')
    return padded

def resample_to_8bit(nda, cmin=0, cmax=2**16):
    """
    Resample a 16 bit image to 8 bit, optionally bracketing a given intensity range
    :param nda: np.ndarray input data of arbitrary dimension
    :param cmin: intensity level on 16-bit scale that become zero in 8-bit scale
    :param cmax: intensity level on 16-bit scale that become maximum (255) in 8-bit scale
    :return: rescaled data of same dimension as input
    """
    return rescale_intensity(
        np.clip(nda, cmin, cmax),
        in_range=(cmin, cmax + 1),
        out_range=(0, 2**8)
    ).astype('uint8')


def rescale(nda, clip=0.0):
    """
    Rescale an image for a given clipping ratio
    :param nda: input data of arbitrary dimension and scale
    :param clip: Ratio of clipping in the resulting image
    :return: rescaled image of same dimension as input
    """
    clip_pct = (100.0 * clip, 100.0 * (1.0 - clip))
    cmin, cmax = np.percentile(nda, clip_pct)
    rescaled = rescale_intensity(nda, in_range=(cmin, cmax))
    return rescaled